@@ -11,7 +11,6 @@ import (
 
 	"github.com/aws/aws-sdk-go/aws"
 	"github.com/golang/mock/gomock"
-	openshiftapiv1 "github.com/openshift/api/config/v1"
 	log "github.com/sirupsen/logrus"
 	"github.com/stretchr/testify/assert"
 
@@ -27,13 +26,13 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 
-	machineapi "github.com/openshift/cluster-api/pkg/apis/machine/v1beta1"
 	autoscalingv1 "github.com/openshift/cluster-autoscaler-operator/pkg/apis/autoscaling/v1"
 	autoscalingv1beta1 "github.com/openshift/cluster-autoscaler-operator/pkg/apis/autoscaling/v1beta1"
+	machineapi "github.com/openshift/machine-api-operator/pkg/apis/machine/v1beta1"
 
-	"github.com/openshift/hive/pkg/apis"
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
-	hivev1aws "github.com/openshift/hive/pkg/apis/hive/v1/aws"
+	"github.com/openshift/hive/apis"
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
+	hivev1aws "github.com/openshift/hive/apis/hive/v1/aws"
 	"github.com/openshift/hive/pkg/constants"
 	"github.com/openshift/hive/pkg/controller/remotemachineset/mock"
 	"github.com/openshift/hive/pkg/remoteclient"
@@ -617,6 +616,33 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 				*testClusterAutoscaler("1"),
 			},
 		},
+		{
+			name:              "Create machine autoscalers with zero minReplicas",
+			clusterDeployment: testClusterDeployment(),
+			machinePool:       testAutoscalingMachinePool(0, 5),
+			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
+				testClusterAutoscaler("1"),
+			},
+			generatedMachineSets: []*machineapi.MachineSet{
+				testMachineSet("foo-12345-worker-us-east-1a", "worker", false, 0, 0),
+				testMachineSet("foo-12345-worker-us-east-1b", "worker", false, 0, 0),
+				testMachineSet("foo-12345-worker-us-east-1c", "worker", false, 0, 0),
+			},
+			expectedRemoteMachineSets: []*machineapi.MachineSet{
+				testMachineSet("foo-12345-worker-us-east-1a", "worker", false, 0, 0),
+				testMachineSet("foo-12345-worker-us-east-1b", "worker", false, 0, 0),
+				testMachineSet("foo-12345-worker-us-east-1c", "worker", false, 0, 0),
+			},
+			expectedRemoteMachineAutoscalers: []autoscalingv1beta1.MachineAutoscaler{
+				*testMachineAutoscaler("foo-12345-worker-us-east-1a", "1", 0, 2),
+				*testMachineAutoscaler("foo-12345-worker-us-east-1b", "1", 0, 2),
+				*testMachineAutoscaler("foo-12345-worker-us-east-1c", "1", 0, 1),
+			},
+			expectedRemoteClusterAutoscalers: []autoscalingv1.ClusterAutoscaler{
+				*testClusterAutoscaler("1"),
+			},
+		},
 	}
 
 	for _, test := range tests {
@@ -632,8 +658,8 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 			if test.machinePool != nil {
 				localExisting = append(localExisting, test.machinePool)
 			}
-			fakeClient := fake.NewFakeClient(localExisting...)
-			remoteFakeClient := fake.NewFakeClient(test.remoteExisting...)
+			fakeClient := fake.NewClientBuilder().WithRuntimeObjects(localExisting...).Build()
+			remoteFakeClient := fake.NewClientBuilder().WithRuntimeObjects(test.remoteExisting...).Build()
 
 			mockCtrl := gomock.NewController(t)
 			defer mockCtrl.Finish()
@@ -661,7 +687,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 				},
 				expectations: controllerExpectations,
 			}
-			_, err := rcd.Reconcile(reconcile.Request{
+			_, err := rcd.Reconcile(context.TODO(), reconcile.Request{
 				NamespacedName: types.NamespacedName{
 					Name:      fmt.Sprintf("%s-worker", testName),
 					Namespace: testNamespace,
@@ -817,7 +843,7 @@ func testMachineSpec(machineType string) machineapi.MachineSpec {
 		log.WithError(err).Fatal("error encoding AWS machine provider spec")
 	}
 	return machineapi.MachineSpec{
-		ObjectMeta: metav1.ObjectMeta{
+		ObjectMeta: machineapi.ObjectMeta{
 			Labels: map[string]string{
 				"machine.openshift.io/cluster-api-cluster":      testInfraID,
 				"machine.openshift.io/cluster-api-machine-role": machineType,
@@ -927,6 +953,9 @@ func testClusterDeployment() *hivev1.ClusterDeployment {
 			Namespace:  testNamespace,
 			Finalizers: []string{hivev1.FinalizerDeprovision},
 			UID:        types.UID("1234"),
+			Labels: map[string]string{
+				constants.VersionMajorMinorPatchLabel: "4.4.0",
+			},
 		},
 		Spec: hivev1.ClusterDeploymentSpec{
 			ClusterName: testName,
@@ -950,11 +979,6 @@ func testClusterDeployment() *hivev1.ClusterDeployment {
 				Type:   hivev1.UnreachableCondition,
 				Status: corev1.ConditionFalse,
 			}},
-			ClusterVersionStatus: &openshiftapiv1.ClusterVersionStatus{
-				Desired: openshiftapiv1.Update{
-					Version: "4.4.0",
-				},
-			},
 		},
 	}
 }
@@ -966,3 +990,11 @@ func printAWSMachineProviderConfig(cfg *awsprovider.AWSMachineProviderConfig) st
 	}
 	return string(b)
 }
+
+func withClusterVersion(cd *hivev1.ClusterDeployment, version string) *hivev1.ClusterDeployment {
+	if cd.Labels == nil {
+		cd.Labels = make(map[string]string, 1)
+	}
+	cd.Labels[constants.VersionMajorMinorPatchLabel] = version
+	return cd
+}
